<?php
define('ONE_DAY', 24*3600);

$ar3 = array(
      array(1, "Device busy"),
      array(2, "Fatal error"),
      array(2, "Printer paper end"),
      array(2, "CMOS reset indication"),
      array(3, "Printer online"),
      array(1, "User operation in progress"),
      array(3, "Fiscal unit online"),
      array(3, "Battery good"),
      array(2, "(Reserved)"),
      array(0, "Open day"),
      array(1, "Signature in progress"),
      array(1, "Recover in progress"),
      array(2, "(Reserved)"),
      array(1, "Fiscal warning indicator"),
      array(1, "Reached maximum number of day signatures"),
      array(1, "Fiscal file used to store daily data after a Z closure is now full")
   );
   
$LIMITS = array();
limits_reinit($x2, $LIMITS);

function limits_reinit($x2, &$LIMITS)
{
   $LIMITS = array(
      1 =>(int)@$x2->Agp['TXC_ZLIMIT'], 
      2 =>(int)@$x2->Agp['TXC_ZLIMITFASY'], 
      3 => (int)@$x2->Agp['TXC_ZLIMITPOS'],
      9999 => (int)@$x2->Agp['TXC_QUERYSPAN']*60 // minutes -> seconds
      );
      
   for ($i=1;$i<=3;$i++) {
      if ($LIMITS[$i]<=0) {
         $LIMITS[$i]=1700;
      }
   }
   
   if ($LIMITS[9999]<=0) {
      $LIMITS[9999] = 3600; // 1 hour
   }
}

class cMACHINETYPE
{
   private $_mt=0;
   
   function __construct($mt=0)
   {
      $this->reset($mt);
   }
   
   function reset($mt)
   {
      $this->_mt=$mt;
   }
   
   function ToString()
   {
      $s1='Unknown'; // default 
      
      switch ($this->_mt)
      {
      case 1:
         $s1 = 'DM100/1000';
         break;
      case 2:
         $s1 = 'FASY';
         break;
      case 3:
         $s1 = 'POS';
         break;
      default:
         $s1 = 'Unknown';
         break;
      }
      return $s1;
   } 
}

class cTXCERROR
{
   private $_s1;
   private $ms11;
   private $location='';
   
   private $evt_lastupdate=false;
   private $evt_1stsign=false;
   private $evt_timelastz=false;
   private $evt_zoverlimit=false; 
   private $evt_zmissing=false;
   private $evt_harderror=false;
   private $evt_apperror=false;
   
   private $t1='taxi_alert';
   private $prim='MS11,ALRTYPE';
   private $queryAlrt="SELECT ALRTYPE, LASTSEND FROM taxi_alert WHERE MS11=? AND AGENTID=?";
   
   function __construct($o1=null)
   {
      $this->_s1 = array();
      
      if ($o1!==null) {
         $this->parse($o1);
      }
   }
    
   function parse($o1)
   {
      global $ar3, $LIMITS;
      
      $this->ms11=$o1->MS11;
      
      if (isset($o1->TXLOCATION)) {
         $this->location=$o1->TXLOCATION;
      }
           
      $tnow=time();
         
      $s2=$o1->LASTUPDATE;
      $told = (int)DateTimeFromString($s2);
      if (($tnow - $told) >= $LIMITS[9999]) { // more than 1 hour or so...
         $this->Add("More then 1 hour since last update");
         $this->evt_lastupdate=true;
      }
      
      $s2=$o1->FIRSTSIGNINZ;
      if (!empty($s2) && !startsWith('00', $s2) && $o1->MDAILY>1) {
         $told = (int)DateTimeFromString($s2);
         if (($tnow - $told)>=ONE_DAY) { // more than 1 day (24 hours)
            $this->Add("More then 24 hours since 1st signature");
            $this->evt_1stsign=true;
         }
      }
      
      $s2=$o1->LASTZTAKEN;
      if (!empty($s2) && !startsWith('00', $s2)) {
         $told = (int)DateTimeFromString($s2);
         if (($tnow - $told)>=ONE_DAY) { // more than 1 day (24 hours)
            $this->Add("More then 24 hours since last Z");
            $this->evt_timelastz=true;
         }
      }
      else if ($o1->LASTZNR>1) {
         $this->Add("Missing previous Z");
         $this->evt_zmissing=true;
      } 
      
      $matype = empty($o1->MACHINETYPE)?1:(int)$o1->MACHINETYPE;
      $LIMITS=(int)@$LIMITS[$matype];   
      if ($o1->LASTZNR >= $LIMITS) {
         $this->Add("Last Z reached or exceeded ".$LIMITS);
         $this->evt_zoverlimit=true;
      }

      // Check status
      $stx = $o1->MLASTHARDST; // device status
      if (!is_null($stx)) {
         for ($i=0;$i<8;$i++) {
            $x = $stx & (1<<$i);
            $ec = $ar3[$i][0];
            
            // 6:Attention, 7:checked-error, 4:checked-ok, 8:not-checked-error, 5:not-checked-ok  
            // TaxSpooler: node3->Item[i]->ImageIndex = x?(ec==1?6:(ec==2?7:4)):(ec==3?8:5);
            if ($x) {
               if ($ec==1) {
                  $this->Add($ar3[$i][1]." ATTENTION");
                  $this->evt_harderror=true;
               }
               else if ($ec==2) {
                  $this->Add($ar3[$i][1]." CHECKED");
                  $this->evt_harderror=true;
               }
            }
            else {
               if ($ec==3) {
                  $this->Add($ar3[$i][1]." NOT CHECKED");
                  $this->evt_harderror=true;
               } 
            }
         }
      }

      $stx = $o1->MLASTAPPST; // app status
      if (!is_null($stx)) {
         for ($i=0;$i<8;$i++) {
            $x = $stx & (1<<$i);
            $ec = $ar3[$i+8][0];
            
            if ($x) {
               if ($ec==1) {
                  $this->Add($ar3[$i+8][1]." ATTENTION");
                  $this->evt_apperror=true;
               }
               else if ($ec==2) {
                  $this->Add($ar3[$i+8][1]." CHECKED");
                  $this->evt_apperror=true;
               }
            }
            else {
               if ($ec==3) {
                  $this->Add($ar3[$i+8][1]." NOT CHECKED");
                  $this->evt_apperror=true;
               } 
            }
         }
      }
   }
   
   function Add($s1)
   {
      $this->_s1[]=$s1;
   }
   
   function Get()
   {
      return implode(' + ', $this->_s1);
   }
   
   function Alert($x2, $AllowEmail)
   {
      $db=&$x2->db;
      
      /*
      // needed for mail()
      $s1=ini_get("sendmail_from");
      if (empty($s1)) {
         echo "[sendmail_from] not defined!";
         return;
      }
      */
      
      $timespan = (int)@$x2->Agp['TXC_ALRTSPAN'];
      if ($timespan<=0) {
         $timespan = 60; // 60 minutes
      }
      $timespan *= 60; // convert into seconds
      
      $tnow = time();
      $nowstr = date('Y-m-d H:i:s');
      
      $alerts=array();
      $data=array($this->ms11, $x2->agentid);
      if (($rs=$db->Execute($this->queryAlrt, $data))===false) {
         echo $db->ErrorMsg();
         return;
      }
      while ($o1=$rs->FetchNextObject()) { 
         $ar1=explode(':', strtr($o1->LASTSEND,' -','::') );
         if (count($ar1)==6 && $ar1[0]!=='0000') {
            // mktime(hour,minute,second,month,day,year)
            $lastsend = (int)mktime($ar1[3], $ar1[4], $ar1[5], $ar1[1], $ar1[2], $ar1[0]);
         }
         else {
            $lastsend=0;
         }
         $alerts[$o1->ALRTYPE] = $lastsend;
      }
      $rs->Close();
      
      $evt=array(); // names of events triggered
      $evX=0; 
      
      if ($this->evt_lastupdate && !empty($x2->Agp[$e1='TXC_EVT_LASTUPDATE'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1]; 
         }
      }
      
      if ($this->evt_1stsign=false && !empty($x2->Agp[$e1='TXC_EVT_1STSIGN'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      }
      
      if ($this->evt_timelastz && !empty($x2->Agp[$e1='TXC_EVT_TIMELASTZ'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      } 
      
      if ($this->evt_zoverlimit && !empty($x2->Agp[$e1='TXC_EVT_ZOVERLIMIT'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      } 
      
      if ($this->evt_zmissing && !empty($x2->Agp[$e1='TXC_EVT_ZMISSING'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      }
      
      if ($this->evt_harderror && !empty($x2->Agp[$e1='TXC_EVT_HARDERROR'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      }
      
      if ($this->evt_apperror && !empty($x2->Agp[$e1='TXC_EVT_APPERROR'])) {
         if (($tnow - (int)@$alerts[$e1])>=$timespan) {
            $evt[] = $e1;
            $evX |= $x2->Agp[$e1];
         }
      }
      
      //
      // Send alerts
      //
      $upd=false;
      
      if ($AllowEmail && ($evX & 1)!=0 && !empty($x2->Agp['TXC_EMAILS'])) {
         $message = $this->location."\r\n".$this->ms11."\r\n".$this->Get();
         echo $message.' '.strlen($message)."\n";

         // $x2->Agp['TXC_EMAILS']: comma separated list of recipients
         $toList=$x2->Agp['TXC_EMAILS'];
         
         $mail = new Zend_Mail('UTF-8');
         $mail->setBodyText($message);
         $mail->setSubject("TaxCenter event");
         
         $ar1=explode(',', $toList);
         foreach ($ar1 as $to1) {
            $mail->addTo($to1);
         }
         
         try {
            $mail->Send();
            $upd=true;
         }
         catch (Exception $e1) {
            echo "Email error: ".$e1;
         }
      }
      
      if (($evX & 2)!=0 && !empty($x2->Agp['TXC_SMSLIST'])) {
         $message = 'TaxCenter '.$this->ms11.': '.$this->Get();
         $l=strlen($message);
         
         $maxL = cSMS_SEND::MaxAsciiPart * cSMS_SEND::MaxParts;
         
         $c1=new cSMS_SEND();
         $c1->show_ui=false;
         $c1->as_is=true; // english only
         
         if ($l<($maxL)) {
            $c1->smstext=$message;
            if ($c1->send( $x2, $x2->Agp['TXC_SMSLIST'] )) {
               $upd=true;
            }
         }
         else {
            $n=count($this->_s1);
            $message = 'TaxCenter '.$this->ms11.': '.$this->_s1[0];
            for ($i=1;$i<$n;$i++) {
               if (strlen($message) + strlen($this->_s1[$i]) + 3 <= $maxL) {
                  $message .= ' + '.$this->_s1[$i];
               }
               else {
                  $c1->smstext=$message;
                  if ($c1->send( $x2, $x2->Agp['TXC_SMSLIST'] )) {
                     $upd=true;
                  }
                  // reset
                  $message = 'TaxCenter '.$this->ms11.': '.$this->_s1[$i];
               }
            }
            $c1->smstext=$message;
            if ($c1->send( $x2, $x2->Agp['TXC_SMSLIST'] )) {
               $upd=true;
            }
         }
      }
         
      if ($upd) {
         foreach ($evt as $alrtype) {
            $data=array(
               'MS11' => $this->ms11,
               'ALRTYPE' => $alrtype,
               'AGENTID' => $x2->agentid,
               'LASTSEND' => $nowstr
            );  
            if (!ReplaceEx($db, $this->t1, $data, $this->prim))
               echo __LINE__.$db->ErrorMsg();
         }
      }
   }
}
?>